// Encapsulate SOAP Binding that extends WSDL and SOAPCall object
function SOAPMauer(port, binding, operation, brand) { // for WSDL 1.1 / SOAP 1.1 or 1.2(???)
  this.port = port;
  this.binding = binding;
  this.operation = operation;
  this.brand = brand;

  this.soapCall = null;
  this.soapHeaderBlocks = null;
  this.soapParameters = null;

  // Create the call instance
  this.soapCall = new SOAPCall(); // nsISOAPCallve
  this.soapEncoding = new SOAPEncoding();
  this.soapCall.encoding = this.soapEncoding;
  //soapEncoding = soapEncoding.getAssociatedEncoding(NS_SOAP_1_1_ENCODING, false);

  var definitions = brand.xmldocument.documentElement; // root element of the WSDL Document

  // The purpose of the SOAP binding element is to signify that the binding is bound
  // to the SOAP protocol format: Envelope, Header and Body.
  var soapbinding = findNextSiblingNS(binding.firstChild, [NS_WSDL_1_1_SOAP_1_1_BINDING, NS_WSDL_1_1_SOAP_1_2_BINDING], "binding");
  var style = "document"; // default for all operations
  if(soapbinding.hasAttribute("style")) {
    style = soapbinding.getAttribute("style");
    // [BUG] Although the value of the required transport attribute indicates which transport
    // of SOAP this binding corresponds to, we deside which transport protocol to use via
    // soapCall.transportURI URI
  }

  var NS_WSDL_SOAP_BINDING = soapbinding.namespaceURI; // namespace for goven version (SOAP 1.1 or 1.2)
  // determine SOAP version
  this.version = this.soapCall.VERSION_1_1; // default
  if(NS_WSDL_SOAP_BINDING == NS_WSDL_1_1_SOAP_1_2_BINDING) {
    this.version = this.soapCall.VERSION_1_2;
  }

  // Load additional schema types from XML Schema
  var schemaLoader = this.soapEncoding.schemaCollection; // the schema collection of all associated encodings
  for(var types = findNextSibling(definitions.firstChild, NS_WSDL_1_1, "types"); types != null; types = findNextSibling(types.nextSibling, NS_WSDL_1_1, "types")) {
    for(var schema = findNextSiblingNS(types.firstChild, [NS_SCHEMA_2001, NS_SCHEMA_1999], "schema"); schema != null; schema = findNextSiblingNS(schema.nextSibling, [NS_SCHEMA_2001, NS_SCHEMA_1999], "schema")) {
      try {
        schemaLoader.processSchemaElement(schema);
      }
      catch(ex) {
        dump(ex + "\n");
      }
    }
  }

  // The SOAP address binding is used to give a port an address (a URI).
  // A port using the SOAP binding MUST specify exactly one address.
  // [BUG] The URI scheme specified for the address must correspond
  // to the transport specified by the soap:binding.
  var soapaddress = findNextSibling(port.firstChild, NS_WSDL_SOAP_BINDING, "address");
  this.soapCall.transportURI = soapaddress.getAttribute("location");

  this.to = this.soapCall.transportURI;
  this.from = "undefined";

  // An operation element within a binding specifies binding information for the operation
  // with the same name within the binding's portType.
  var soapoperation = findNextSibling(operation.firstChild, NS_WSDL_SOAP_BINDING, "operation");
  if(soapoperation.hasAttribute("soapAction"))
    this.soapCall.actionURI = soapoperation.getAttribute("soapAction");
  if(soapoperation.hasAttribute("style")) // this overrides default style ('document') and default style for binding
    style = soapoperation.getAttribute("style");

  // The input element specifies the concrete grammar for the request message format
  var input = findNextSibling(operation.firstChild, NS_WSDL_1_1, "input");

  // The soap:body element specifies how the message parts appear inside the SOAP Body element.
  var soapbody = findNextSibling(input.firstChild, NS_WSDL_SOAP_BINDING, "body");

  // If the style is RPC, find the method name and target object URI
  // If it is document-style, these are both left blank
  this.methodName = "";
  this.targetObjectURI = "";
  if (style == "rpc") {
    this.methodName = operation.getAttribute("name");
    if(soapbody.hasAttribute("namespace"))
      this.targetObjectURI = soapbody.getAttribute("namespace");
  }

  var portType = findNextSiblingByName(definitions.firstChild, NS_WSDL_1_1, "portType", QName2LocalName(binding.getAttribute("type")));
  var portTypeOperation = findNextSiblingByName(portType.firstChild, NS_WSDL_1_1, "operation", operation.getAttribute("name"));
  var parameterOrder; // parameter order within an operation
  if(portTypeOperation.hasAttribute("parameterOrder"))
    parameterOrder = portTypeOperation.getAttribute("parameterOrder");
  var portTypeInput = findNextSibling(portTypeOperation.firstChild, NS_WSDL_1_1, "input");
  var messageQName = portTypeInput.getAttribute("message");

  var message = findNextSiblingByName(definitions.firstChild, NS_WSDL_1_1, "message", QName2LocalName(messageQName));

  this.soapHeaderBlocks = new Array();
  for(var soapheader = findNextSibling(input.firstChild, NS_WSDL_SOAP_BINDING, "header"); soapheader != null; soapheader = findNextSibling(soapheader.nextSibling, NS_WSDL_SOAP_BINDING, "header")) {
    var headermessage = findNextSiblingByName(definitions.firstChild, NS_WSDL_1_1, "message", QName2LocalName(soapheader.getAttribute("message")));
    var headerpart = findNextSiblingByName(headermessage.firstChild, NS_WSDL_1_1, "part", soapheader.getAttribute("part"));

    var soapHeaderBlock = new SOAPHeaderBlock();

    // Get the name, namespaceURI and type of the block
    this._processPart(soapheader, headerpart, soapHeaderBlock);

    // XXX Need a way to specify that a block should not be
    // encoded.
    var encodingStyle = soapheader.getAttribute("encodingStyle");
    if(encodingStyle == null) encodingStyle = "";
    if(soapheader.getAttribute("use") == "encoded") {
      if(soapheader.hasAttribute("encodingStyle")) {
        soapHeaderBlock.encoding = this.soapEncoding.getAssociatedEncoding(encodingStyle, false);
      }
    }

    this.soapHeaderBlocks.push(soapHeaderBlock);
  }
  // [TODO]
  //this.soapHeaderBlocks[0].value = {string: "test", 'int': 1};
  //this.soapHeaderBlocks[1].value = {'int': 2};

  this.soapParameters = new Array();
  var encodingStyle = soapbody.getAttribute("encodingStyle");
  if(encodingStyle == null) encodingStyle = "";
  var soapbodyparts = null;
  if(soapbody.hasAttribute("parts"))
    soapbodyparts = soapbody.getAttribute("parts");

  for(var part = findNextSibling(message.firstChild, NS_WSDL_1_1, "part"); part != null; part = findNextSibling(part.nextSibling, NS_WSDL_1_1, "part")) {
    if(soapbodyparts == null || soapbodyparts.indexOf(part.getAttribute("name")) != -1) {
      var soapParameter = new SOAPParameter();

      // Get the name, namespaceURI and type of the block
      this._processPart(soapbody, part, soapParameter);

      // XXX Need a way to specify that a block should not be
      // encoded.
      if(soapbody.getAttribute("use") == "encoded") {
        if(encodingStyle) {
          soapParameter.encoding = this.soapEncoding.getAssociatedEncoding(encodingStyle, false);
        }
      }

      this.soapParameters.push(soapParameter);
    }
  }
  // [TODO]
  //this.soapParameters[0].value = "test";
  //this.soapParameters[0].value = [{varString: "test", varInt: "1", varFloat: "0.5"}, {varString: "test1", varInt: "2", varFloat: "1.5"}];
}
var globe = null;
// Get the name, namespaceURI and type of the block based on
// information from the WSDL part. If the schema component
// associated with the part is an element, these values come
// from the schema description of the element. If it is a
// type, then the values are gathered from elsewhere.
SOAPMauer.prototype._processPart = function(soapheaderorbody, part, soapBlock) {
  if(part.hasAttribute("element")) {
    var elementName = QName2LocalName(part.getAttribute("element"));
    var elementNamespaceURI = QName2NamespaceURI(part, part.getAttribute("element"));
    var schemaElement = this.soapEncoding.schemaCollection.getElement(elementName, elementNamespaceURI);

    if(schemaElement != null) {
      soapBlock.name = schemaElement.name;
      soapBlock.schemaType = schemaElement.type;
      soapBlock.namespaceURI = schemaElement.targetNamespace;
    }

  }
  else if(part.hasAttribute("type")) {
    var typeName = QName2LocalName(part.getAttribute("type"));
    var typeNamespaceURI = QName2NamespaceURI(part, part.getAttribute("type"));
    var schemaType = this.soapEncoding.schemaCollection.getType(typeName, typeNamespaceURI);
    if(schemaType != null) {
      soapBlock.schemaType = schemaType;
    }

    soapBlock.name = part.getAttribute("name");
    soapBlock.namespaceURI = soapheaderorbody.getAttribute("namespace");
  }
  // else error
}

SOAPMauer.prototype.getBlocks = function() {
  return this.soapHeaderBlocks.concat(this.soapParameters);
}

SOAPMauer.prototype.send = function(mauerListener) {
  // get parameters'/headers' values from content
  this.encodeBody();

  this.soapCall.encode(this.version, this.methodName, this.targetObjectURI, 0, null, this.soapParameters.length, this.soapParameters);

  //netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
  //this.soapCall.asyncInvoke(this);
  this.request = new XMLHttpRequest();
  this.request.open("POST", this.soapCall.transportURI, true);

  // set request headers
  if(this.version == this.soapCall.VERSION_1_1)
    this.request.setRequestHeader("Content-Type", "text/xml; charset=UTF-8");
  else if(this.version == this.soapCall.VERSION_1_2)
    this.request.setRequestHeader("Content-Type", "application/soap+xml; charset=UTF-8");

  // Apache Axis web services WSDL files say to set soapAction to "" and require it to be sent.  
  // So only check if its not void instead of using AStringIsNull.
  if ("actionURI" in this.soapCall && this.version == this.soapCall.VERSION_1_1) {

    //XXX necko doesn't allow empty header values, so set it to " "
    if (this.soapCall.actionURI.length == 0)
      this.soapCall.actionURI = " ";

    this.request.setRequestHeader("SOAPAction", this.soapCall.actionURI);
  }

  this.request.onreadystatechange = mauerListener;

  //request.send(new XMLSerializer().serializeToString(this.soapCall.message));
  this.request.send(this.soapCall.message);
}

SOAPMauer.prototype.getAllHeaders = function() {
  var header = "";

  // set request headers
  if(this.version == this.soapCall.VERSION_1_1)
    header += "Content-Type: text/xml; charset=UTF-8\r\n";
  else if(this.version == this.soapCall.VERSION_1_2)
    header += "Content-Type: application/soap+xml; charset=UTF-8\r\n";

  // Apache Axis web services WSDL files say to set soapAction to "" and require it to be sent.  
  // So only check if its not void instead of using AStringIsNull.
  if ("actionURI" in this.soapCall && this.version == this.soapCall.VERSION_1_1) {

    //XXX necko doesn't allow empty header values, so set it to " "
    if (this.soapCall.actionURI.length == 0)
      this.soapCall.actionURI = " ";

    header += "SOAPAction: " + this.soapCall.actionURI + "\r\n";
  }

  return header;
}

// return encoded text
SOAPMauer.prototype.encode = function() {
  this.encodeBody();

  this.soapCall.encode(this.version, this.methodName, this.targetObjectURI, 0, null, this.soapParameters.length, this.soapParameters);

  return this.getAllHeaders() + "\r\n" + (new XMLSerializer().serializeToString(this.soapCall.message));
}

// display content of the SOAP interface
SOAPMauer.prototype.decode = function(dbody) {
  this.decodeBody(dbody);
}


/*SOAPMauer.prototype.encodeBody = function() {
  // get parameters'/headers' values from content
  var blocks = this.getBlocks();

  var inputs = this.block.getElementsByTagName("input");

  for(var i = 0; i < blocks.length; i++) {
    var str = inputs[i].value;
    if(str.match(/^\s*\{.+\}\s*$/))
      str = "a=" + str;
    var value = eval(str);

    try {
      blocks[i].value = value;
    }
    catch(ex) {
      blocks[i].value = inputs[i].value;
    }
  }

}

SOAPMauer.prototype.decodeBody = function(dbody) {
  this.block = dbody;

  // clear content
  while(dbody.firstChild)
    dbody.removeChild(dbody.firstChild);

  var blocks = this.getBlocks();

  for(var i = 0; i < blocks.length; i++) {
    var ad = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
    var blockName = document.createElementNS("http://www.w3.org/1999/xhtml", "span");
    blockName.setAttribute("partName", blocks[i].name);
    blockName.appendChild(document.createTextNode(blocks[i].name + ": " + blocks[i].schemaType.name + " "));
    ad.appendChild(blockName);
    var blockValue = document.createElementNS("http://www.w3.org/1999/xhtml", "input");
    blockValue.setAttribute("type", "text");
    ad.appendChild(blockValue);

    dbody.appendChild(ad);

  }

}*/

SOAPMauer.prototype.encodeBody = function() {
  // get parameters'/headers' values from content
  var blocks = this.getBlocks();

  var start = this.block;

  var inputVars = new Object();
  var array = new Array();
  for(var ch = start.firstChild;ch!=null;ch=ch.nextSibling)
  {
    //simple----------------------------------------------------
    if (ch.getAttribute("name")=="simple"){
      val = ch.getElementsByTagName("div")[0].getElementsByTagName("input")[0].value;
      inputVars = val;
	
    }
    else
    //array of struct-------------------------------------------
    if (ch.getAttribute("name")=="arrofstruct")	{
      for(var chh = ch.firstChild;chh!=null;chh=chh.nextSibling) {
        inputVars[chh.getElementsByTagName("span")[0].getAttribute("partName")] = chh.getElementsByTagName("input")[0].value;
      }
      array.push(inputVars);
      inputVars = new Object();
    }
    else
    //struct----------------------------------------------------
    if (ch.getAttribute("name")=="struct") {
      for(var chh = ch.firstChild;chh!=null;chh=chh.nextSibling) {
        inputVars[chh.getElementsByTagName("span")[0].getAttribute("partName")] = chh.getElementsByTagName("input")[0].value;
      }
    }
    else
    
    
    //simple array----------------------------------------------
    if (ch.getAttribute("name")=="simplearray") {
      array.push(ch.getElementsByTagName("input")[0].value);
      //alert(ch.getElementsByTagName("input")[0].value)

    }

  }

  blocks[0].value = inputVars;
  if(array.length!=0) blocks[0].value = array;

}

SOAPMauer.prototype.decodeBody = function(dbody) {
  this.block = dbody;

  // clear content
  while(dbody.firstChild)
    dbody.removeChild(dbody.firstChild);

  var blocks = this.getBlocks();

  for(var i = 0; i < blocks.length; i++) {
    
    /*for(io in blocks[i].schemaType)
		alert(io +"  "+blocks[i].schemaType[io]);*/
    
    types(blocks[i],1,dbody,false);
  }

}

//====================================================================================================
function simpletype(node,d,dbody,incompl,isArr,isClone,isStructUs) {
  var ad = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
  ad.setAttribute("class", "simpleType");
  var blockName = document.createElementNS("http://www.w3.org/1999/xhtml", "span");
  blockName.setAttribute("partName", node.name);
	var minOccurs = "";
	//for(ii in node)
	//alert(ii+" "+node[ii]);
	if(node.minOccurs==1)minOccurs = "(requied field)";
  if(incompl=="false")
    blockName.appendChild(document.createTextNode(node.name + ": " + node.schemaType.name + " "+minOccurs));
  else if (incompl=="true")
    blockName.appendChild(document.createTextNode(node.name + ": " + node.type.name + " "+minOccurs));
  else 
    blockName.appendChild(document.createTextNode(node.name + ": " + node.arrayType.name + " "+minOccurs));

  ad.appendChild(blockName);
  var blockValue = document.createElementNS("http://www.w3.org/1999/xhtml", "input");
  blockValue.setAttribute("type", "text");
  blockValue.setAttribute("name",d);
  ad.appendChild(blockValue);
  if(!isArr&&!isStructUs) {
    var adnew = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
    adnew.setAttribute("name","simple");
    adnew.appendChild(ad);
    dbody.appendChild(adnew);
  }

  return ad;
}

function complextype(node,d,dbody,isClone) {
  ss = node.schemaType;
  isArr = null;

  //for(i in node.schemaType.arrayType)
    //alert(i+" "+node.schemaType.arrayType[i] +" "+typeof(node.schemaType.arrayType[i])+"\n");
				
  var adiv = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
  if(!isClone) {
    var ad = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
    var blockName = document.createElementNS("http://www.w3.org/1999/xhtml", "span");
    blockName.setAttribute("partName", node.name);
    blockName.appendChild(document.createTextNode(node.name + ": " + node.schemaType.name + " "));
    ad.appendChild(blockName);

    if(ss.isArray) {
      var addButton = document.createElementNS("http://www.w3.org/1999/xhtml", "input");
      addButton.setAttribute("type","button");
      addButton.setAttribute("value","Add element");
      addButton.setAttribute("name","Add");
      var ell = new createElListener(node);
      addButton.addEventListener("click", ell, false);
      ad.appendChild(addButton);
    }

    dbody.appendChild(ad);
  }

  var model;
  var isa = ss.isArray;
  if(ss.isArray) {
    ss=ss.arrayType;
    isArr = true;
  }

  if(ss.contentModel==2 || ss.contentModel==3 || ss.contentModel==4) { //something inside
    model = ss.modelGroup;
    /*
    for(io in model)
		alert(io +"  "+model[io]);*/
    
    //alert(model)
    countt = model.particleCount;
    c=0;
    names = new Array();

    while(c<countt) {
      if(model.getParticle(c).particleType==1)
        divEl = simpletype(model.getParticle(c),(d+1)+""+c,dbody,"true",isArr,false,true);
      adiv.appendChild(divEl);
      c++;
    }

    if(isa)
      adiv.setAttribute("name","arrofstruct");
    else
      adiv.setAttribute("name","struct");
      
    //  if(model.compositor==2||model.compositor==3) adiv.setAttribute("name","group");

    dbody.appendChild(adiv);
  } 
  else { //simple array
    ss = node.schemaType;
    divEl = simpletype(ss,(d+1),dbody,"simple",isArr);
    adiv.setAttribute("name","simplearray");
    adiv.appendChild(divEl);
    dbody.appendChild(adiv);
  }

  if(isa) {
    var remButton = document.createElementNS("http://www.w3.org/1999/xhtml", "input");
    remButton.setAttribute("type","button");
    remButton.setAttribute("value","Remove element");
    remButton.setAttribute("name","remove");
    remButton.setAttribute("onclick","removeEl(event)");
    divEl.appendChild(remButton);
  }

}

function removeEl(ev) {
  var toDelete = ev.target.parentNode.parentNode;
  var par = ev.target.parentNode.parentNode.parentNode;
  par.removeChild(toDelete);
}

function types(node,d,dbody,isClone) {

  if(node.schemaType.schemaType==1) simpletype(node,d,dbody,"false",false,isClone);
  if(node.schemaType.schemaType==2) complextype(node,d,dbody,isClone);
}

function createElListener(a) {
  this.a = a;
}

createElListener.prototype.handleEvent = function(evt) {
  var divo = evt.target.parentNode.parentNode;	
  types(this.a,1,divo,true);
}
