const NS_SCHEMA_2001 = "http://www.w3.org/2001/XMLSchema";
const NS_SCHEMA_1999 = "http://www.w3.org/1999/XMLSchema";
const NS_WSDL_1_1 = "http://schemas.xmlsoap.org/wsdl/";
const NS_WSDL_1_1_SOAP_1_1_BINDING = "http://schemas.xmlsoap.org/wsdl/soap/";
const NS_WSDL_1_1_SOAP_1_2_BINDING = "http://www.w3.org/2003/01/wsdl/soap12"; // wsdl1.2? (http://www.w3.org/2003/01/wsdl)
const NS_WSDL_1_1_HTTP_BINDING = "http://schemas.xmlsoap.org/wsdl/http/";
const NS_WSDL_1_1_MIME_BINDING = "http://schemas.xmlsoap.org/wsdl/mime/";
const NS_SOAP_1_1_ENCODING = "http://schemas.xmlsoap.org/soap/encoding/";
const NS_SOAP_1_2_ENCODING = "http://www.w3.org/2001/09/soap-encoding";

function WSDLBrand(URL, scstream, brandListener) {
  this.brandListener = brandListener;
  this.from = URL;
  this.to = "undefined";

  // load from stream (scriptable input stream)
  var data = "";
  while (scstream.available()) {
    data += scstream.read(scstream.available());
  }

  // parse string into DOM tree
  var parser = new DOMParser();
  this.xmldocument = parser.parseFromString(data, "text/xml");
  this.xmltext = data;

  if(this.xmldocument.documentElement.nodeName == "parsererror") { // error
    throw "error parsing text/xml in DOMParser.parseFromString";
  }
}

// create interface to invoke operations
WSDLBrand.prototype.encode = function() {
  return this.xmltext;
}

// create interface to invoke operations
WSDLBrand.prototype.decode = function(dbody) {
  var curPos = 0;

  // clear content
  while(dbody.firstChild)
    dbody.removeChild(dbody.firstChild);

  var definitions = this.xmldocument.documentElement; // root element of the WSDL Document

  for(var service = findNextSibling(definitions.firstChild, NS_WSDL_1_1, "service"); service != null; service = findNextSibling(service.nextSibling, NS_WSDL_1_1, "service")) {
    var port = findNextSibling(service.firstChild, NS_WSDL_1_1, "port");
    if(!port)
      continue;
    var binding = findNextSiblingByName(definitions.firstChild, NS_WSDL_1_1, "binding", QName2LocalName(port.getAttribute("binding")));
    var bindingText = this.xmltext.match(new RegExp("<[^>]*binding.+name\s*=\s*[\"\']" + binding.getAttribute("name") + "[\"\'][^<]*>", ""))[0];
    var bindingPos = this.xmltext.indexOf(bindingText);

    var type = "unknown";
    if(findNextSibling(binding.firstChild, NS_WSDL_1_1_SOAP_1_1_BINDING, "binding"))
      type = "SOAP 1.1";
    else if(findNextSibling(binding.firstChild, NS_WSDL_1_1_SOAP_1_2_BINDING, "binding"))
      type = "SOAP 1.2";
    else if(findNextSibling(binding.firstChild, NS_WSDL_1_1_HTTP_BINDING, "binding"))
      type = "HTTP 1.1";

    for(var operation = findNextSibling(binding.firstChild, NS_WSDL_1_1, "operation"); operation != null; operation = findNextSibling(operation.nextSibling, NS_WSDL_1_1, "operation")) {
      var operationText = this.xmltext.substring(bindingPos + bindingText.length).match(new RegExp("<[^>]*operation.+name\s*=\s*[\"\']" + operation.getAttribute("name") + "[\"\'][^<]*>[\r\n]*", ""))[0];

      var operationPos = this.xmltext.indexOf(operationText, bindingPos);

      dbody.appendChild(document.createTextNode(this.xmltext.substring(curPos, operationPos)));
      var operationSpan = document.createElementNS("http://www.w3.org/1999/xhtml", "span");
      operationSpan.setAttribute("class", "operation");
      operationSpan.onclick = this.brandListener;
      operationSpan.setAttribute("portQName", port.getAttribute("name"));
      operationSpan.setAttribute("operationQName", operation.getAttribute("name"));
      operationSpan.appendChild(document.createTextNode(this.xmltext.substr(operationPos, operationText.length)));
      dbody.appendChild(operationSpan);
      var curPos = operationPos + operationText.length;


    }
  }

  dbody.appendChild(document.createTextNode(this.xmltext.substring(curPos)));

}

WSDLBrand.prototype.encodeOperation = function(portQName, operationQName) {
  var definitions = this.xmldocument.documentElement; // root element of the WSDL Document
  var portName = QName2LocalName(portQName);
  var operationName = QName2LocalName(operationQName);

  // A port defines an individual endpoint by specifying a single address for a binding.
  // The name attribute provides a unique name among all ports defined within in the enclosing WSDL document.
  // The binding attribute (of type QName) refers to the binding using the linking rules defined by WSDL.
  var port = null;
  for(var service = findNextSibling(definitions.firstChild, NS_WSDL_1_1, "service"); service != null; service = findNextSibling(service.nextSibling, NS_WSDL_1_1, "service")) {
    var port = findNextSiblingByName(service.firstChild, NS_WSDL_1_1, "port", portName);
  }

  // A binding defines message format and protocol details for operations and
  // messages defined by a particular portType.
  // There may be any number of bindings for a given portType.
  // A binding references the portType that it binds using the type attribute.

  // [BUG] It seem that some WSDL authors eliminate prefixes
  // from qualified names in attribute values, assuming that
  // the names will resolve to the targetNamespace, while
  // they should technically resolve to the default namespace.
  var binding = findNextSiblingByName(definitions.firstChild, NS_WSDL_1_1, "binding", QName2LocalName(port.getAttribute("binding")));

  // An operation element within a binding specifies binding information for
  // the operation with the same name within the binding's portType.
  var operation = findNextSiblingByName(binding.firstChild, NS_WSDL_1_1, "operation", operationName);

  // [TODO] Select corresponding extention depending on namespace
  // of the given binding elements which extend WSDL

  // The purpose of the SOAP binding element is to signify that the binding is bound
  // to the SOAP protocol format: Envelope, Header and Body.
  if(findNextSiblingNS(binding.firstChild, [NS_WSDL_1_1_SOAP_1_1_BINDING, NS_WSDL_1_1_SOAP_1_2_BINDING], "binding") != null)
    return new SOAPMauer(port, binding, operation, this);

  return null;
}

// find next sibling of the DOMElement with given localName in namespaceURI
function findNextSibling(element, namespaceURI, localName) {
  for(var si = element; si != null; si = si.nextSibling) {
    if(si.namespaceURI == namespaceURI && si.localName == localName)
      return si;
  }
  return null;
}

// find next sibling of the DOMElement with given localName in namespaceURI from nsArray
function findNextSiblingNS(element, nsArray, localName) {
  var namespaces = String(nsArray);
  for(var si = element; si != null; si = si.nextSibling) {
    if(si.localName == localName && namespaces.indexOf(si.namespaceURI) != -1)
      return si;
  }
  return null;
}

// find next sibling of the DOMElement with given localName in namespaceURI
// with the attribute 'name' equal to name
function findNextSiblingByName(element, namespaceURI, localName, name) {
  for(var si = element; si != null; si = si.nextSibling) {
    if(si.namespaceURI == namespaceURI && si.localName == localName && si.getAttribute("name") == name)
      return si;
  }
  return null;
}

// find next sibling of the DOMElement with given localName in namespaceURI from nsArray
// with the attribute 'name' equal to name
function findNextSiblingByNameNS(element, nsArray, localName, name) {
  var namespaces = String(nsArray);
  for(var si = element; si != null; si = si.nextSibling) {
    if(si.localName == localName && namespaces.indexOf(si.namespaceURI) != -1 && si.getAttribute("name") == name)
      return si;
  }
  return null;
}

// get namespaceURI from QName in the scope of DOMElement element
function QName2NamespaceURI(element, qName) {
  var pos = qName.indexOf(":");
  if(pos == -1)
    return element.lookupNamespaceURI("");
  return element.lookupNamespaceURI(qName.substring(0, pos));
}

// get localName from QName
function QName2LocalName(qName) {
  var pos = qName.indexOf(":");
  if(pos == -1) return qName;
  return qName.substring(pos + 1);
}
